#ifndef __KVS_VISCLIENT_COMMUNICATION_CONTROLLER_H__
#define __KVS_VISCLIENT_COMMUNICATION_CONTROLLER_H__

#include "kvs/RGBColor"
#include "kvs/glut/Application"
#include "kvs/glut/Screen"
#include "kvs/glut/PushButton"
#include "kvs/glut/Slider"
#include "kvs/glut/CheckBox"
#include "ProgressBar.h"

#include "kvs/glut/RadioButton"
#include "kvs/glut/RadioButtonGroup"

namespace
{
const kvs::RGBColor Gray( 100, 100, 100 );
const kvs::RGBColor Red( 255, 0, 0 );
const kvs::RGBColor Green( 0, 255, 0 );
const kvs::RGBColor Blue( 0, 0, 255 );
kvs::RGBColor Color = Gray;
}

namespace kvs
{
namespace visclient
{

class Command;

class LastStepCheckBox : public kvs::glut::CheckBox
{
public:

    LastStepCheckBox( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::CheckBox( screen ),
        command( command )
    { this->setState( true ); };
    
    void stateChanged( void );

private:
    Command* command;
};

class StartDetailedAnalysisButton : public kvs::glut::PushButton
{
public:

    StartDetailedAnalysisButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::PushButton( screen ),
        command( command )
    {};

    void pressed( void );

private:
    Command* command;
};

class StartAbstractAnalysisButton : public kvs::glut::PushButton
{
public:

    StartAbstractAnalysisButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::PushButton( screen ),
        command( command )
    {};

    void pressed( void );

private:
    Command* command;
};

class StopButton : public kvs::glut::PushButton
{
private:
    Command* command;
public:

    StopButton( kvs::glut::Screen* screen, Command* command ) :
        kvs::glut::PushButton( screen ),
        command( command )
    {};

    void pressed( void );
};

class ShowHideButton : public kvs::glut::PushButton
{
private:
    Command* command;
    bool isShown;
public:

    ShowHideButton( kvs::glut::Screen* screen, Command* command ) :
        kvs::glut::PushButton( screen ),
        command( command ),
        isShown( true )
    {};

    void pressed( void );
};

class TimeStepSlider : public kvs::glut::Slider
{
public:

    TimeStepSlider( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::Slider( screen ),
        command( command )
    {};

    void sliderMoved( void );
    void sliderReleased( void );

private:
    Command* command;
};

class TimeSegmentMinSlider : public kvs::glut::Slider
{
public:

    TimeSegmentMinSlider( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::Slider( screen ),
        command( command ) {};

    void sliderMoved( void );

private:
    Command* command;
};

class TimeSegmentMaxSlider : public kvs::glut::Slider
{
public:

    TimeSegmentMaxSlider( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::Slider( screen ),
        command( command ) {};

    void sliderMoved( void );

private:
    Command* command;
};

class RandomBasedShuffleButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    RandomBasedShuffleButton( kvs::glut::Screen* screen, Command* command  ):
        kvs::glut::RadioButton( screen ),
        command( command ) {};

    void stateChanged( void );
};

class IndexBasedSingleShuffleButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    IndexBasedSingleShuffleButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command ) {};

    void stateChanged( void );
};

class IndexBasedCPUParallelShuffleButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    IndexBasedCPUParallelShuffleButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command ) {};

    void stateChanged( void );
};

class IndexBasedGPUParallelShuffleButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    IndexBasedGPUParallelShuffleButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command ) {};

    void stateChanged( void );
};

class SummalizedTransferButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    SummalizedTransferButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};


    void stateChanged( void );
};

class DividedTransferButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    DividedTransferButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};


    void stateChanged( void );
};

class UniformSamplingButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    UniformSamplingButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};


    void stateChanged( void );
};

class RejectionSamplingButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    RejectionSamplingButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};


    void stateChanged( void );
};

class MetropolisSamplingButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    MetropolisSamplingButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};


    void stateChanged( void );
};

class AllNodesButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    AllNodesButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};


    void stateChanged( void );
};

class SlaveNodesButton : public kvs::glut::RadioButton
{
private:
    Command* command;

public:

    SlaveNodesButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};


    void stateChanged( void );
};

class MisesRenderingButton : public kvs::glut::RadioButton
{
public:

    MisesRenderingButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};

    void stateChanged();

private:
    Command* command;
};

class UXRenderingButton : public kvs::glut::RadioButton
{
public:

    UXRenderingButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};

    void stateChanged();

private:
    Command* command;
};

class UYRenderingButton : public kvs::glut::RadioButton
{
public:

    UYRenderingButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};

    void stateChanged();

private:
    Command* command;
};

class UZRenderingButton : public kvs::glut::RadioButton
{
public:

    UZRenderingButton( kvs::glut::Screen* screen, Command* command ):
        kvs::glut::RadioButton( screen ),
        command( command )
    {};

    void stateChanged();

private:
    Command* command;
};

class Controller : public kvs::glut::WidgetBase
{
public:
    // MODIFIED START fp)m.tanaka 2013.09.01
//	Controller( kvs::glut::Screen* screen, Command* command );
    Controller( kvs::glut::Screen* screen, kvs::glut::Screen* screen3, Command* command );
    // MODIFIED END   fp)m.tanaka 2013.09.01

public:
    kvs::glut::Screen* screen;
// tantan
    kvs::glut::Screen* screen3;
    Command* command;

public:
    std::auto_ptr<StartDetailedAnalysisButton> detail;
    std::auto_ptr<StartAbstractAnalysisButton> abst;
    std::auto_ptr<StopButton> stop;
    std::auto_ptr<ShowHideButton> showHide;
    std::auto_ptr<kvs::glut::RadioButtonGroup> group1;
    std::auto_ptr<kvs::glut::RadioButtonGroup> group2;
    std::auto_ptr<RandomBasedShuffleButton> randomBased;
    std::auto_ptr<IndexBasedSingleShuffleButton> indexBasedSingle;
    std::auto_ptr<IndexBasedCPUParallelShuffleButton> indexBasedCPUParallel;
    std::auto_ptr<IndexBasedGPUParallelShuffleButton> indexBasedGPUParallel;
    std::auto_ptr<SummalizedTransferButton> summalized;
    std::auto_ptr<DividedTransferButton> divided;
    std::auto_ptr<kvs::glut::RadioButtonGroup> samplingGroup;
    std::auto_ptr<UniformSamplingButton> uniform;
    std::auto_ptr<RejectionSamplingButton> rejection;
    std::auto_ptr<MetropolisSamplingButton> metropolis;
    std::auto_ptr<kvs::glut::RadioButtonGroup> nodeGroup;
    std::auto_ptr<AllNodesButton> allnodes;
    std::auto_ptr<SlaveNodesButton> slavenodes;
    std::auto_ptr<kvs::glut::RadioButtonGroup> renderingGroup;
    std::auto_ptr<MisesRenderingButton> misesRenderingButton;
    std::auto_ptr<UXRenderingButton> uxRenderingButton;
    std::auto_ptr<UYRenderingButton> uyRenderingButton;
    std::auto_ptr<UZRenderingButton> uzRenderingButton;
};



}
}

#endif
