#include <kvs/Message>
#include <kvs/StructuredVolumeObject>
#include <kvs/StructuredVolumeImporter>
#include <kvs/glut/Application>
#include <kvs/glut/Screen>
#include <kvs/glut/LegendBar>
#include <kvs/Date>
#include <kvs/Time>
#include <kvs/CommandLine>
#include <kvs/ParticleVolumeRenderer>
#include <kvs/KVSMLObjectPoint>
#include <kvs/PointExporter>
#include <kvs/PointImporter>
#include <stdio.h>
#include "TimerEvent.h"

extern kvs::visclient::TimerEvent* g_timer_event;
extern char ValueKeyFrameFile[256];
extern int ValueNumKeyFrames;
extern void redraw_AniCont();

static std::vector<kvs::Xform> m_xforms;
static std::vector<int> m_timeSteps;

static size_t write_float( FILE* fp, float val )
{
    return fwrite( &val, sizeof( float ), 1, fp );
}

static size_t read_float( FILE* fp, float* val )
{
    return fread( val, sizeof( float ), 1, fp );
}

static size_t write_int( FILE* fp, int val )
{
    return fwrite( &val, sizeof( int ), 1, fp );
}

static size_t read_int( FILE* fp, int* val )
{
    return fread( val, sizeof( int ), 1, fp );
}

// Initialize Xform
void KeyFrameAnimation_init()
{
    m_xforms.clear();
    m_timeSteps.clear();
    ValueNumKeyFrames = m_xforms.size();
    // redraw_AniCont();
}


// Add Xform
int KeyFrameAnimation_add()
{
    g_timer_event->disableKeyFrameAnimation();
    kvs::PointObject* object = g_timer_event->getPointObject();
    if ( object )
    {
        kvs::Xform xform = object->xform();
        m_xforms.push_back( xform );
        m_timeSteps.push_back( g_timer_event->getTimeStep() );
    }

    ValueNumKeyFrames = m_xforms.size();
    redraw_AniCont();
    return m_xforms.size();
}

// Animation start
void KeyFrameAnimation_start()
{
    g_timer_event->enableKeyFrameAnimation();
}

// Animation stop
void KeyFrameAnimation_stop()
{
    g_timer_event->disableKeyFrameAnimation();
}

// Animation switch start/stop
void KeyFrameAnimation_toggle()
{
    g_timer_event->toggleKeyFrameAnimation();
}

// delete all Xform
void KeyFrameAnimation_delete()
{
    m_xforms.clear();
    m_timeSteps.clear();
    ValueNumKeyFrames = m_xforms.size();
    redraw_AniCont();
}

// delete last Xform
int KeyFrameAnimation_erase()
{
    if ( m_xforms.size() > 1 )
    {
        m_xforms.pop_back();
        m_timeSteps.pop_back();
    }

    ValueNumKeyFrames = m_xforms.size();
    redraw_AniCont();
    return m_xforms.size();
}

// write Xform data;
void KeyFrameAnimation_write()
{
    FILE* fp = fopen( ValueKeyFrameFile, "w" );
    if ( fp )
    {
        for ( size_t i = 0; i < m_xforms.size(); i++ )
        {
            write_int( fp, m_timeSteps[i] );
            for ( int j = 0; j < 3; j++ )
            {
                write_float( fp, m_xforms[i].rotation()[j].x() );
                write_float( fp, m_xforms[i].rotation()[j].y() );
                write_float( fp, m_xforms[i].rotation()[j].z() );
            }
            write_float( fp, m_xforms[i].translation().x() );
            write_float( fp, m_xforms[i].translation().y() );
            write_float( fp, m_xforms[i].translation().z() );
            write_float( fp, m_xforms[i].scaling().x() );
            write_float( fp, m_xforms[i].scaling().y() );
            write_float( fp, m_xforms[i].scaling().z() );
        }
        fclose( fp );
    }
    else
    {
        fprintf( stderr, " KeyFrameAnimation: Can't open file %s.\n", ValueKeyFrameFile );
    }
}

// read Xform data
int KeyFrameAnimation_read()
{
    g_timer_event->disableKeyFrameAnimation();
    FILE* fp = fopen( ValueKeyFrameFile, "r" );
    if ( fp )
    {
        m_xforms.clear();
        while ( 1 )
        {
            int   ival;
            float fval[3];
            if ( read_int( fp, &ival ) )
            {
                m_timeSteps.push_back( ival );
                read_float( fp, &fval[0] );
                read_float( fp, &fval[1] );
                read_float( fp, &fval[2] );
                kvs::Vector3<float> vec0 = kvs::Vector3<float>( fval );
                read_float( fp, &fval[0] );
                read_float( fp, &fval[1] );
                read_float( fp, &fval[2] );
                kvs::Vector3<float> vec1 = kvs::Vector3<float>( fval );
                read_float( fp, &fval[0] );
                read_float( fp, &fval[1] );
                read_float( fp, &fval[2] );
                kvs::Vector3<float> vec2 = kvs::Vector3<float>( fval );
                kvs::Matrix33f rotation = kvs::Matrix33f( vec0, vec1, vec2 );
                read_float( fp, &fval[0] );
                read_float( fp, &fval[1] );
                read_float( fp, &fval[2] );
                kvs::Vector3<float> translation = kvs::Vector3<float>( fval );
                read_float( fp, &fval[0] );
                read_float( fp, &fval[1] );
                read_float( fp, &fval[2] );
                kvs::Vector3<float> scaling = kvs::Vector3<float>( fval );
                kvs::Xform xform = kvs::Xform( translation, scaling, rotation );

                m_xforms.push_back( xform );

            }
            else
            {
                fclose( fp );
                break;
            }
        }
        ValueNumKeyFrames = m_xforms.size();
        redraw_AniCont();
        // Animation start
        g_timer_event->enableKeyFrameAnimation();
    }
    else
    {
        fprintf( stderr, " KeyFrameAnimation: Can't open file %s.\n", ValueKeyFrameFile );
        return -1;
    }

    return 0;
}

// get vector of Xforms
std::vector<kvs::Xform>* KeyFrameAnimation_getXforms()
{
    return &m_xforms;
}

// get vector of timestep
std::vector<int>* KeyFrameAnimation_getTimeSteps()
{
    return &m_timeSteps;
}

