#include "kvs/Message"
#include "kvs/PointImporter"
#include "kvs/PointObject"
#include "kvs/PointRenderer"
#include "kvs/glut/Application"
#include "kvs/glut/Screen"
#include "kvs/TransferFunction"
#include "kvs/glut/LegendBar"
#include "kvs/glut/OrientationAxis"
#include "kvs/File"
#include "kvs/ExtractEdges"
#include "kvs/glut/Slider"
#include "kvs/PolygonRenderer"
#include "kvs/HydrogenVolumeData"
#include "kvs/Timer"

#include "Command.h"
#include "ParticleServer.h"
#include "TransferFunctionEditor.h"
#include "TransferFunctionEditorMain.h" // APPEND fj) 2015.03.04
#include "Controller.h"
#include "PVRenderer.h"
#include "TimerEvent.h"
#include "Argument.h"
#include "Label.h"
#include "LegendpanelUI.h" // APPEND by FEAST 2016.1.18
#include "KeyPressEvent.h"
#include "ParticleMerger.h"

#include "kvs/Thread"
#include "kvs/Mutex"

#include "TransferFunctionEditorMain.h"
// APPEND START fp)m.tanaka 2013.09.01
#include "v2head.h"
#include "v3head.h"
#include "AnimationControl.h"
// APPEND END   fp)m.tanaka 2013.09.01
#include "MemoryMonitor.h"
//#include <string>
#include "sighandler.h"

// APPEND START BY)M.Tanaka 2015.03.03
#include "FileParam.h"
kvs::visclient::StepLabel* g_label;
kvs::visclient::FPSLabel* g_fpsLabel;
kvs::visclient::LegendBar* g_legend;
kvs::glut::OrientationAxis* g_orientation_axis;
// APPEND END   BY)M.Tanaka 2015.03.03
// APPEND START FP)M.TANAKA 2015.03.16
kvs::PointObject* g_control_object;
kvs::visclient::TimerEvent* g_timer_event;
// APPEND END   FP)M.TANAKA 2015.03.16

#if 1 // FEAST modify 20160128
int g_curStep = -1;
int lavel_time_step= -1;
#endif // FEAST modify 20160128 end

kvs::visclient::TransferFunctionEditorMain* ptfe_main; // APPEND Fj 2015.03.05
kvs::visclient::ParticleMergerUI m_Particle_UI; // APPEND FEAST 2016.2.5
kvs::visclient::LegendpanelUI m_Legend_UI; // APPEND FEAST 2016.2.8
static bool show_aui = false; // APPEND FEAST 2016.2.5

int main( int argc, char** argv )
{
    int ret;
    kvs::visclient::Argument argument( argc, argv );
    kvs::visclient::ParticleServer server;
    kvs::visclient::Command command( &server );
    kvs::visclient::ComThread comthread( &command );

    setCropParam( &command.param.paramCrop );
    init_sighandler();

// APPEND START fp)m.tanaka 2013.09.01
    g_command = &command;
//	kvs::visclient::Command command3( &server);
// APPEND END   fp)m.tanaka 2013.09.01

// APPEND START BY)M.Tanaka 2015.03.03
    if ( argument.repeat_level_detailed_f )
    {
        repeatleveld = argument.repeat_level_detailed;
    }
    if ( argument.subpixel_level_detailed_f )
    {
        subpixelleveld = argument.subpixel_level_detailed;
    }
// APPEND START BY)M.Tanaka 2015.03.11
    if ( argument.samplingType_f )
    {
        if ( strcmp( argument.samplingType.c_str(), "u" ) == 0 )
        {
//printf("***** samplingtype:u\n");
            samplingtypelevel = kvs::visclient::PBVRParam::UniformSampling;
        }
        else if ( strcmp( argument.samplingType.c_str(), "r" ) == 0 )
        {
//printf("***** samplingtype:r\n");
            samplingtypelevel = kvs::visclient::PBVRParam::RejectionSampling;
        }
        else if ( strcmp( argument.samplingType.c_str(), "m" ) == 0 )
        {
//printf("***** samplingtype:m\n");
            samplingtypelevel = kvs::visclient::PBVRParam::MetropolisSampling;
        }
    }
    if ( argument.detailedparticletransferType_f )
    {
        if ( strcmp( argument.detailedparticletransfertype.c_str(), "all" ) == 0 )
        {
//printf("***** transfertype:all\n");
            detailedparticletransfertypelevel = kvs::visclient::PBVRParam::Summalized;
        }
        else if ( strcmp( argument.detailedparticletransfertype.c_str(), "div" ) == 0 )
        {
//printf("***** transfertype:div\n");
            detailedparticletransfertypelevel = kvs::visclient::PBVRParam::Divided;
        }
    }
    printf( "\n\n\n***** 0 shading:%s\n", shadinglevel );
    if ( argument.shadingType_f )
    {
        strcpy( shadinglevel, argument.shadingtype.c_str() );
        printf( "***** 1 shading:%s\n", shadinglevel );
    }
// APPEND END   BY)M.Tanaka 2015.03.11

    command.param.detailedRepeatLevel = repeatleveld;
    command.param.detailedSubpixelLevel = subpixelleveld;
// APPEND START BY)M.Tanaka 2015.03.11
    command.param.samplingType = static_cast<kvs::visclient::PBVRParam::SamplingType>( samplingtypelevel );
    command.param.detailedTransferType = static_cast<kvs::visclient::PBVRParam::DetailedParticleTransferType>( detailedparticletransfertypelevel );
// APPEND END   BY)M.Tanaka 2015.03.11

// APPEND END   BY)M.Tanaka 2015.03.03

    command.param.abstractRepeatLevel = argument.repeat_level_abstract;
    command.param.abstractSubpixelLevel = argument.subpixel_level_abstract;
    command.param.transferFunction = argument.transfer_function;

    command.param.hostname = argument.hostname;//default: localhost
    command.param.port = argument.port;//default: 60000
    command.param.shadingType_f = argument.shadingType_f;

    /*
    #if defined(_WIN32)
            resolution_height_device = RESOLUTIONHEIGHT_DEVICE;
    #else
    #ifdef __APPLE__
            char *cmdline1 = "xrandr -q | grep maximum |awk '{print $14}'";
    #else
    */
#if not defined(WIN32) && not defined(__APPLE__)
    char* cmdline1 = "timeout 5 xrandr -q | grep maximum |awk '{print $14}'";
//#endif
    FILE* fp;
    char buf[128];
    fp = popen( cmdline1, "r" );
    fgets( buf, 128, fp );
    pclose( fp );
    int nitem = sscanf( buf, "%d", &resolution_height_device );
//        std::cout<<"nitem " << nitem << std::endl;
//        std::cout<<"resolution_height_device " << resolution_height_device << std::endl;
    if ( nitem < 1 )
    {
        std::cout << "No X-Server." << std::endl;
        exit( 1 );
    }
#endif
#if 0
    if ( resolution_height_level > resolution_height_device - DOCK_H )
    {
        resolution_height_level = resolution_height_device - DOCK_H;
    }
#endif
    kvs::glut::Application app( argc, argv );
    kvs::glut::Screen screen( &app );

    char buff_volumedata[100] = "/work/filter/binary08/binary";

    if ( argument.server_subvol_dir != "" )
    {
        command.param.inputdir = argument.server_subvol_dir;
    }
    else
    {
        command.param.inputdir = buff_volumedata;
    }
// APPEND START BY)M.Tanaka 2015.03.03
    strcpy( pfi_path_server, command.param.inputdir.c_str() );
// APPEND END   BY)M.Tanaka 2015.03.03

    command.isImportTransferFunctionParam = true;
    std::cout<<"main.cpp:l196:CSmode="<<command.param.CSmode<<std::endl;
    std::cout<<"main.cpp:l197:maxServerTimeStep="<<command.param.maxServerTimeStep<<std::endl;

    //内部でsendとrecvを実行
    //最初の送信(client->daemon)
    //最初の受信(daemon->client)
    command.getServerParticleInfomation( command.param, &command.result );
    std::cout<<"main.cpp:L203"<<std::endl;
    command.isImportTransferFunctionParam = false;
    //command.paramの中にdaomonから受信した伝達関数がコピーされている

    std::cout<<"main.cpp:L206"<<std::endl;
    command.param.paramExTransFunc.show();

    if ( argument.particle_limit_f )
    {
        plimitlevel = argument.particle_limit;
    }
    if ( argument.particle_density_f )
    {
        pdensitylevel = argument.particle_density;
    }
    if ( argument.viewer_f )
    {
        sscanf( argument.viewer.c_str(), "%fx%f", &resolution_width_level, &resolution_height_level );
    }
    if ( argument.particle_data_size_limit_f )
    {
        particle_data_size_limit_input = argument.particle_data_size_limit;
    }

    command.param.particle_limit = plimitlevel;
    command.param.particle_density = pdensitylevel ;
    command.param.particle_data_size_limit = particle_data_size_limit_input;
// MODIFIED START BY)M.Tanaka 2015.03.03
//  screen.setGeometry( 0, 0, 620, 620 );
    int i_w = ( int )( ( long )resolution_width_level );
    int i_h = ( int )( ( long )resolution_height_level );
    screen.setGeometry( 0, 0, i_w, i_h );
// MODIFIED END   BY)M.Tanaka 2015.03.03

    command.screen = &screen;
    command.param.camera = screen.camera();

// APPEND START fp)m.tanaka 2013.09.01
#include "v2newpanel.h"
    if ( argument.param_file != "" )
    {
        FileParam::ReadParamFile( argument.param_file.c_str(), 0 );
    }
//	Win3vd();
//	extfedit.create_window();
    kvs::visclient::TransferFunctionEditorMain tfe( &command, &screen );
    ptfe_main = &tfe; // Add Fj 2015.03.05
    tfe.show();
    tfe.hide(); // Add FEAST 2016.2.5
    // APPEND START Fj 2015.03.16
    if ( argument.param_file != "" )
    {
        ptfe_main->importFileWithInit( argument.param_file.c_str() );
    }
    else
    {
        std::cout<<"main.cpp:L258"<<std::endl;
        command.param.paramExTransFunc.show();
        //command.paramの中の伝達関数を、伝達関数エディタに渡す
        ptfe_main->importExtendTransferFunction( command.param.paramExTransFunc );
    }

    if ( argument.tffile_f )
    {
        ptfe_main->importFileWithInit( argument.tffile_name );
    }
    // APPEND END Fj 2015.03.16
// APPEND END   fp)m.tanaka 2013.09.01
// APPEND START fp)m.tanaka 2013.09.01
    g_screen = &screen;
// APPEND END   fp)m.tanaka 2013.09.01
//jupiter
//    g_volumedata->set_text( command.param.inputdir.c_str() );

    if ( argument.header != "" )
    {
        command.param.particledir = argument.header;
        command.param.CSmode = 0;
    }

    // pin引数を反映してUIを作成
    for (int i = 0; i < 10; i++)
    {
        if (argument.pin_dir[i].empty() != true)
        {
            command.param.particle_merge_param.particles[i + 1].filepath.append( argument.pin_dir[i] );
        }
    }
    m_Particle_UI.createPanel( command.param.CSmode, &command.param.particle_merge_param );
    // 粒子の表示・非表示情報を内部変数に反映
    m_Particle_UI.setParam( &command.param.particle_merge_param );
    command.ParticleMergUI = &m_Particle_UI;
    // Serverのstep範囲情報を得る
    ptfe_main->apply();

    // pin引数で指定されたファイルを全てチェックしてstepの範囲を得る
    kvs::visclient::ParticleMerger tmp_merger;
    tmp_merger.setParam( command.param.particle_merge_param, command.param.minServerTimeStep, command.param.maxServerTimeStep );
    /*
    // 表示するものだけフラグを立てる(表示するファイルだけstep範囲の情報がほしいため)
    for (int i = 0; i < 10; i++)
    {
        if (argument.pin_dir[i].empty() != true)
        {
            tmp_merger.setenable( i + 1 );
        }
    }
    */
    int local_time_start = tmp_merger.getMergedInitialTimeStep();
    int local_time_end = tmp_merger.getMergedFinalTimeStep();

// MODIFIED START  fp)m.tanaka 2013.09.01
    command.initabstractParticles( command.param, &command.result, local_time_start, local_time_end );
//    command.generateAbstractParticles( command.param, &command.result, local_time_start, local_time_end ); /* modified 20140123 */
    command.generateDetailedParticles();
//	command3.generateAbstractParticles();
//	command3.generateDetailedParticles();
// MODIFIED END    fp)m.tanaka 2013.09.01
//#ifndef __APPLE__
    Win3vd();
    std::cout << " ### Win3vd() done " << std::endl;
    if ( argument.param_file != "" )
    {
        FileParam::updateWin3v( argument.param_file.c_str() );
    }
//#endif
    FILEpanelCreate();

    kvs::glut::Screen screen2( &app );
    screen2.setTitle( "Communication" );
// MODIFIED START  fp)m.tanaka 2013.09.01
    //screen2.setGeometry( 0, 620, 512, 620 );
    //screen2.setGeometry( 0, 620, 256, 250 );
    screen2.setGeometry( 0, 620, 286, 250 );
// MODIFIED END    fp)m.tanaka 2013.09.01
    //screen2.show(); // Comment out by Fj 2015.03.04

// MODIFIED START  fp)m.tanaka 2013.09.01
//	kvs::visclient::Controller controller( &screen2, &command );
    kvs::visclient::Controller controller( &screen2, &screen3, &command );
// tantan
    command.screen3 = &screen3;
// MODIFIED END    fp)m.tanaka 2013.09.01
    controller.show();

    command.controller = &controller;

// Comment Out by Fj 20150304
//	kvs::visclient::TransferFunctionEditor transferFunctionEditor(&screen);
//	kvs::StructuredVolumeObject* dummyObject = new kvs::HydrogenVolumeData( kvs::Vector3ui(32, 32, 32) );
//	transferFunctionEditor.setVolumeObject( dummyObject );
//	transferFunctionEditor.show();

    command.startDetailedAnalysis(); /* same as press detaild button 2014.07.21 */

// MODIFY START fp)m.tanaka 2013.09.01
//	std::auto_ptr<kvs::visclient::ProgressBar> progress555( new kvs::visclient::ProgressBar( &screen ) );
    std::auto_ptr<kvs::visclient::ProgressBar> progress( new kvs::visclient::ProgressBar( &screen3 ) );
// MODIFY END   fp)m.tanaka 2013.09.01
    progress->setX( 0 );
    progress->setY( 0 );
    progress->setWidth( 500 );
    progress->setValue( 0 );
    progress->setCaption( "Progress" );
    progress->setTextColor( kvs::RGBColor( 0, 0, 0 ) );
//delete show (jupiter)
//  progress->show();

// MODIFY START fp)m.tanaka 2013.09.01
//	std::auto_ptr<kvs::visclient::TimeStepSlider> timeSlider( new kvs::visclient::TimeStepSlider( &screen, &command ) );
    std::auto_ptr<kvs::visclient::TimeStepSlider> timeSlider( new kvs::visclient::TimeStepSlider( &screen3, &command ) );
// MODIFY END   fp)m.tanaka 2013.09.01
    timeSlider->setX( 0 );
// MODIFY START fp)K.YAJIMA 2015.03.16
    //timeSlider->setY( 50 );
    timeSlider->setY( 70 );
// MODIFY END fp)K.YAJIMA 2015.03.16
    timeSlider->setWidth( 500 );
    timeSlider->setValue( 0 );
//	timeSlider->setRange(0,99);
// 	timeSlider->setRange(0,command.param.maxTimeStep); // MODIFY by Fj 2015.03.04
//    timeSlider->setRange( command.param.minTimeStep, command.param.maxTimeStep ); // MODIFY by Fj 2015.03.04
    timeSlider->setRange( 0, 0 ); // MODIFY by Fj 2015.03.04
    timeSlider->setCaption( "Time step" );
    timeSlider->setTextColor( kvs::RGBColor( 0, 0, 0 ) );
    timeSlider->show();

    std::auto_ptr<kvs::visclient::LastStepCheckBox> lastStepCheckBox( new kvs::visclient::LastStepCheckBox( &screen3, &command ) );
    lastStepCheckBox->setX( 50 );
    lastStepCheckBox->setY( 140 );
    lastStepCheckBox->setCaption( "Last step" );
    lastStepCheckBox->setTextColor( kvs::RGBColor( 0, 0, 0 ) );
    lastStepCheckBox->show();
// MODIFY START fp)m.tanaka 2013.09.01
//	std::auto_ptr<kvs::visclient::TimeSegmentMinSlider> segment1( new kvs::visclient::TimeSegmentMinSlider( &screen, &command ) );
    std::auto_ptr<kvs::visclient::TimeSegmentMinSlider> segment1( new kvs::visclient::TimeSegmentMinSlider( &screen3, &command ) );
// MODIFY END   fp)m.tanaka 2013.09.01
    segment1->setX( 0 );
// MODIFY START fp)m.tanaka 2013.09.01
    //segment1->setY( 450 );
// MODIFY START fp)K.YAJIMA 2015.03.16
    //segment1->setY( 100 );
    segment1->setY( 140 );
// MODIFY END fp)K.YAJIMA 2015.03.16
// MODIFY END   fp)m.tanaka 2013.09.01
    segment1->setWidth( 500 );
    segment1->setValue( command.param.minTimeStep );
//	segment1->setRange(0,98);
//	segment1->setRange(0,command.param.maxTimeStep); // MODIFY by Fj 2015.03.04
    segment1->setRange( command.param.minTimeStep, command.param.maxTimeStep ); // MODIFY by Fj 2015.03.04
//	segment1->setCaption("Min Time(Detailed)");
    segment1->setCaption( "Min Time" );
    segment1->setTextColor( kvs::RGBColor( 0, 0, 0 ) );
//delete show (jupiter)
//  segment1->show();

// MODIFY START fp)m.tanaka 2013.09.01
//	std::auto_ptr<kvs::visclient::TimeSegmentMaxSlider> segment2( new kvs::visclient::TimeSegmentMaxSlider( &screen, &command ) );
    std::auto_ptr<kvs::visclient::TimeSegmentMaxSlider> segment2( new kvs::visclient::TimeSegmentMaxSlider( &screen3, &command ) );
// MODIFY END   fp)m.tanaka 2013.09.01

    segment2->setX( 0 );
// MODIFY START fp)m.tanaka 2013.09.01
    //segment2->setY( 500 );
// MODIFY START fp)K.YAJIMA 2015.03.16
    //segment2->setY( 150 );
    segment2->setY( 210 );
// MODIFY END fp)K.YAJIMA 2015.03.16
// MODIFY END   fp)m.tanaka 2013.09.01
    segment2->setWidth( 500 );
//	segment2->setValue(1);
    segment2->setValue( command.param.maxTimeStep );
//	segment2->setRange(1,99);
//	segment2->setRange(0,command.param.maxTimeStep); // MODIFY by Fj 2015.03.04
    segment2->setRange( command.param.minTimeStep, command.param.maxTimeStep ); // MODIFY by Fj 2015.03.04
//	segment2->setCaption("Max Time(Detailed)");
    segment2->setCaption( "Max Time" );
    segment2->setTextColor( kvs::RGBColor( 0, 0, 0 ) );
//delete show (jupiter)
//  segment2->show();

// MODIFY START  fp)h.arai 2015.02.26
    std::auto_ptr<kvs::visclient::StartDetailedAnalysisButton> detail( new kvs::visclient::StartDetailedAnalysisButton( &screen3, &command ) );
    detail->setButtonColor( ::Gray );
    detail->setTextColor( kvs::RGBColor( 255, 255, 255 ) );
    detail->setX( 0 );
    detail->setY( 230 );
    detail->setWidth( 250 );
    detail->setMargin( 10 );
    detail->setCaption( "Detailed analysis" );

    std::auto_ptr<kvs::visclient::StartAbstractAnalysisButton> abst( new kvs::visclient::StartAbstractAnalysisButton( &screen3, &command ) );
    abst->setButtonColor( ::Gray );
    abst->setTextColor( kvs::RGBColor( 255, 255, 255 ) );
    abst->setX( 245 );
    abst->setY( 230 );
    abst->setWidth( detail->width() );
    abst->setMargin( 10 );
    abst->setCaption( "Abstract analysis" );

    std::auto_ptr<kvs::visclient::StopButton> stop( new kvs::visclient::StopButton( &screen3, &command ) );
    if ( command.param.CSmode == 1 )
    {
//jupiter start
#if 0
        stop->setButtonColor( ::Red );
#else
        stop->setButtonColor( ::Gray );
#endif
//jupiter end
    }
    else
    {
        stop->setButtonColor( ::Gray );
    }
    stop->setTextColor( kvs::RGBColor( 255, 255, 255 ) );
    stop->setX( 0 );
// MODIFY START fp)K.YAJIMA 2015.03.16
    //stop->setY( 205 );
    //stop->setY( 285 ); //jpiter
    stop->setY( 0 );
// MODIFY END fp)K.YAJIMA 2015.03.16
    stop->setWidth( abst->width() );
    stop->setMargin( 10 );
    stop->setCaption( "Start/Stop" );
    stop->show();
// MODIFY END  fp)h.arai 2015.02.26
    screen3.show();

    command.progress = progress.get();
    command.lastStepCheckBox = lastStepCheckBox.get();
    command.timeSlider = timeSlider.get();
    command.minTimeSlider = segment1.get();
    command.maxTimeSlider = segment2.get();

    // Create Time Step Label
    kvs::visclient::StepLabel label( &command );
    int yl0 = i_h - I_H_OFFSET;
    label.setPosition( i_w * 1 / 31, yl0 );
    g_label = &label;

    label.setTextColor( kvs::RGBColor( 255, 255, 255 ) );
    label.show();

    kvs::PointObject* object1 = command.abstractParticles[0];

    kvs::ValueArray<kvs::Real32> coords( 3 );
    kvs::RGBColor color( 0, 0, 0 );
    kvs::ValueArray<kvs::Real32> normals( 3 );
    kvs::PointObject* control_object = new kvs::PointObject( coords, color, normals, 1.0 );
    const kvs::Vector3f& min = object1->minObjectCoord();
    const kvs::Vector3f& max = object1->maxObjectCoord();
    control_object->setMinMaxObjectCoords( min, max );
    control_object->setMinMaxExternalCoords( min, max );

// APPEND START FP)M.TANAKA 2015.03.16
    g_control_object = control_object;
// APPEND END   FP)M.TANAKA 2015.03.16

    command.renderer = new kvs::visclient::PVRenderer( object1, command.param.detailedSubpixelLevel, command.param.detailedRepeatLevel );
    if ( !command.renderer )
    {
        kvsMessageError( "Cannot creat a point renderer." );
    }

    kvs::visclient::FPSLabel fpsLabel( &screen, command.renderer );
    fpsLabel.setPosition( i_w * 51 / 124, yl0 );
    fpsLabel.setTextColor( kvs::RGBAColor( 255, 255, 255 ) );
    fpsLabel.show();
    g_fpsLabel = &fpsLabel;

    // For shading.
    command.renderer->disableShading();
// APPEND START BY)M.Tanaka 2015.03.11
    float sd_ka, sd_kd, sd_ks, sd_n;
    if ( strcmp( shadinglevel, "L" ) == 0 )
    {
        printf( "***** shading : L\n" );
        command.renderer->enableShading();
        command.renderer->setShader( kvs::Shader::Lambert() );
    }
    else if ( strcmp( shadinglevel, "P" ) == 0 )
    {
        printf( "***** shading : P\n" );
        command.renderer->enableShading();
        command.renderer->setShader( kvs::Shader::Phong() );
    }
    else if ( strcmp( shadinglevel, "B" ) == 0 )
    {
        printf( "***** shading : B\n" );
        command.renderer->enableShading();
        command.renderer->setShader( kvs::Shader::BlinnPhong() );
    }
    else if ( strncmp( shadinglevel, "L,", 2 ) == 0 )
    {
        command.renderer->enableShading();
        sscanf( &shadinglevel[2], "%f,%f", &sd_ka, &sd_kd );
        printf( "***** shading : L %f %f\n", sd_ka, sd_kd );
        command.renderer->setShader( kvs::Shader::Lambert( sd_ka, sd_kd ) );
    }
    else if ( strncmp( shadinglevel, "P,", 2 ) == 0 )
    {
        command.renderer->enableShading();
        sscanf( &shadinglevel[2], "%f,%f,%f,%f", &sd_ka, &sd_kd, &sd_ks, &sd_n );
        printf( "***** shading : P %f %f %f %f\n", sd_ka, sd_kd, sd_ks, sd_n );
        command.renderer->setShader( kvs::Shader::Phong( sd_ka, sd_kd, sd_ks, sd_n ) );
    }
    else if ( strncmp( shadinglevel, "B,", 2 ) == 0 )
    {
        command.renderer->enableShading();
        sscanf( &shadinglevel[2], "%f,%f,%f,%f", &sd_ka, &sd_kd, &sd_ks, &sd_n );
        printf( "***** shading : B %f %f %f %f\n", sd_ka, sd_kd, sd_ks, sd_n );
        command.renderer->setShader( kvs::Shader::BlinnPhong( sd_ka, sd_kd, sd_ks, sd_n ) );
    }
// APPEND END   BY)M.Tanaka 2015.03.11
#ifndef CPUMODE
    command.renderer->disableLODControl();
#endif
    kvs::visclient::TimerEvent timer_event( &command, &comthread );

    // Legend (Add by FEAST 2016.02.08)
    kvs::visclient::LegendBar legendBar( &screen, &command );
//    legendBar.setOrientation( kvs::glut::LegendBar::OrientationType::Vertical );
    legendBar.setOrientation( kvs::glut::LegendBar::Vertical );
    legendBar.select_tfr( 0 );
    legendBar.setEventType(
        kvs::EventBase::PaintEvent |
        kvs::EventBase::ResizeEvent );
    legendBar.hide();
    g_legend = &legendBar;
    m_Legend_UI.createPanel( &legendBar, &screen );

    // Timer
    int msec = 100;
    std::string interval( "TIMER_EVENT_INTERVAL" );

    if ( getenv( interval.c_str() ) != NULL )
    {
        msec = std::atoi( getenv( interval.c_str() ) );
    }

    kvs::glut::Timer* glut_timer = new kvs::glut::Timer( msec );
    glut_timer->setInterval( msec );
    glut_timer->start();

    command.glutTimer = glut_timer;

    // Create KeyPressEvent
    kvs::visclient::KeyPressEvent key_press_event;

// APPEND START fp)m.takizawa 2014.05.28
//        AnimationControl( &screen );
// APPEND END fp)m.takizawa 2014.05.28

    // Create Data Label
    /*kvs::visclient::SizeLabel size_label( &command );
    size_label.setPosition(420,10);
    size_label.setTextColor( kvs::RGBColor(255,255,255) );
    size_label.show();
    kvs::visclient::NodesLabel nodes_label( &command );
    nodes_label.setPosition(420,30);
    nodes_label.setTextColor( kvs::RGBColor(255,255,255) );
    nodes_label.show();
    kvs::visclient::ElementsLabel elements_label( &command );
    elements_label.setPosition(420,50);
    elements_label.setTextColor( kvs::RGBColor(255,255,255) );
    elements_label.show();*/

    //kvs::TransferFunction& tf = p.transfer_function;

    // Create LegendBar
    /*kvs::glut::LegendBar legend_bar( &screen );
    legend_bar.setColorMap( tf.colorMap() );
    legend_bar.setRange( 0, 1500 );
    legend_bar.setCaption( "Mises stress" );
    legend_bar.setTextColor( kvs::RGBColor(255,255,255) );
    legend_bar.show();*/

    // Create Orientation Box
    kvs::glut::OrientationAxis orientation_axis( &screen );
    g_orientation_axis = &orientation_axis;
    orientation_axis.setPosition( i_w * 53 / 62 - 30, yl0 - 30 );
    orientation_axis.setBoxType( kvs::glut::OrientationAxis::SolidBox );
    orientation_axis.show();

    screen.addKeyPressEvent( &key_press_event );

// APPEND START FP)M.TANAKA 2015.03.16
    g_timer_event = &timer_event;
// APPEND START FP)M.TANAKA 2015.03.16

#ifdef CPUMODE
    std::pair<int, int> id_pair;
    id_pair = screen.registerObject( control_object, command.renderer );
    timer_event.setObject( id_pair.first );
    timer_event.setRenderer( command.renderer );
#else
    screen.registerObject( control_object, command.renderer );
#endif
    screen.addTimerEvent( &timer_event, glut_timer );
    screen.setTitle( "PBVR" );
    //screen.background()->setColor( kvs::RGBColor( 200, 200, 210 ) );
    screen.background()->setColor( kvs::RGBColor( 0, 0, 0 ) );//2018 kawamura
    screen.show();

// APPEND START fp)m.tanaka 2014.05.20
    PaintEvent  paint_event;
    screen.addPaintEvent( &paint_event );

// APPEND START fp)m.takizawa 2014.05.28
//        AnimationControl( &screen );
// APPEND END fp)m.takizawa 2014.05.28

    strcpy( ValueImageFile, argument.image_base_name.c_str() );
// APPEND START fp)m.takizawa 2014.05.28
    AnimationControl( &screen );
// APPEND END fp)m.takizawa 2014.05.28

    glutSetWindow( AnimationControlWinID );
    glutHideWindow();

// System Memory Monitor
    Update_smemory update_smemory;
//   Update_gmemory update_gmemory;
    kvs::Mutex mutex;
    update_smemory.mutex = &mutex;
//   update_gmemory.mutex = &mutex;
    update_smemory.start();
//   update_gmemory.start();
    ret = app.run();
    return ( ret );
}
// APPEND START fp)m.tanaka 2013.09.01
#include "v2foot.h"
// APPEND END   fp)m.tanaka 2013.09.01
